#!/usr/bin/env perl
#
# Copyright (C) 2020 Alexandr Dubovikov (alexandr.dubovikov@gmail.com)
#
# This file is part of liagent
#
# The application checks existing interceptions and if needs, recreate it
#
# checker is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version
#
# checker is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

use 5.010;
use strict;
use warnings;
use DBI;
use POSIX;
use WWW::Curl::Easy;
use JSON;
use Sys::Syslog;                        # all except setlogsock()
    
my $version = "1.2.0";
$| =1;

#recreate all interception
my $FORCE = 0;
my $facility = "info";
my $SYSLOG = 1;
my $program = "liagent-checker";

# Determine path and set default rotation.ini location
my $script_location = `dirname $0`;
$script_location =~ s/^\s+|\s+$//g;
my $default_ini = $script_location."/config.ini";
my $conf_file = $ARGV[0] // $default_ini;
our $CONFIG = read_config($conf_file);

# Optionally load override configuration. perl format
my $rc = "/etc/sysconfig/partrotaterc";
if (-e $rc) {
    do $rc;    
}

if(defined($CONFIG->{"SYSTEM"}{"facility"})) {
  $facility = $CONFIG->{"SYSTEM"}{"facility"};
}
    
if(defined($CONFIG->{"SYSTEM"}{"force"})) {
  $FORCE = $CONFIG->{"SYSTEM"}{"force"};
  $FORCE+=0;
}

if(defined($CONFIG->{"SYSTEM"}{"syslog"})) {
  $SYSLOG = $CONFIG->{"SYSTEM"}{"syslog"};
  $SYSLOG+=0;
}
            


if ($SYSLOG == 1) {
  openlog($program, 'cons,pid', 'user');
  syslog($facility, "starting check.");              
}
 
my %IRECORDS;
my $db = db_connect($CONFIG, "db_interception");
my $query = "SELECT * FROM voip_intercept WHERE deleted=0 AND stop_time > NOW();";
my $sth = $db->prepare($query);
$sth->execute();
my $interceptions = 0;
while(my $ref = $sth->fetchrow_hashref()) {

        my $uuid = $ref->{uuid};
        $IRECORDS{$uuid} = encode_json \%$ref;
        $interceptions++;
}

if ($SYSLOG == 1) { syslog($facility, "Found [".$interceptions."] Interceptions");}

#Check DATA tables
$db = db_connect($CONFIG, "db_interception");
$query = "SELECT * FROM agent_location_session WHERE active=1 AND expire_date > NOW();";
$sth = $db->prepare($query);
$sth->execute();

while(my $ref = $sth->fetchrow_hashref()) {

        my $agent = $ref->{uuid};
        my $host = $ref->{host};
        my $port = $ref->{port};
        my $protocol = $ref->{protocol};
        my $path = $ref->{path};
        my $node = $ref->{node};
        
        if ($SYSLOG == 1) { syslog($facility, "checking agent host:[".$host."], UUID: [".$agent."]");}
        
        foreach my $key (keys %IRECORDS) {        
                check_interception($protocol,$host, $port, $path, $key, $IRECORDS{$key});
        }                
}

if ($SYSLOG == 1) { syslog($facility, "done"); closelog();}
 
exit;

sub db_connect {
    my $CONFIG  = shift;
    my $db_name = shift;

    my $db = DBI->connect("DBI:mysql:".$CONFIG->{"MYSQL"}{$db_name}.":".$CONFIG->{"MYSQL"}{"host"}.":".$CONFIG->{"MYSQL"}{"port"}, $CONFIG->{"MYSQL"}{"user"}, $CONFIG->{"MYSQL"}{"password"});
    return $db;

}
 
 
sub read_config {

    my $ini = shift;

    open (INI, "$ini") || die "Can't open $ini: $!\n";
    my $section;
    my $CONFIG; 
    while (<INI>) {
        chomp;
        if (/^\s*\[(\w+)\].*/) {
            $section = $1;
        }
        if ((/^(.*)=(.*)$/)) {
            my ($keyword, $value) = split(/=/, $_, 2);
            $keyword =~ s/^\s+|\s+$//g;
            $value =~ s/(#.*)$//;
            $value =~ s/^\s+//;  
            $value =~ s/\s+$//;  
            #Debug
            #print "V: [$value]\n";
            $CONFIG->{$section}{$keyword} = $value;
        }
    }
        close(INI);
    return $CONFIG;
}

#check_interception($protocol,$host, $port, $path, $key, $IRECORDS{$key});
sub check_interception {

	my $protocol = shift;
	my $host = shift;
	my $port = shift;
	my $path = shift;
	my $uuid = shift;
	my $data = shift;

	my $curl = WWW::Curl::Easy->new;
 
	$curl->setopt(CURLOPT_HEADER,0);
	my $url = $protocol."://".$host.":".$port."".$path."/".$uuid;
	$curl->setopt(CURLOPT_URL, $url);
 
	# A filehandle, reference to a scalar or reference to a typeglob can be used here.
	my $response_body;
	open(my $fileb, ">", \$response_body);	
	$curl->setopt(CURLOPT_WRITEDATA,$fileb);
 
	# Starts the actual request
	my $retcode = $curl->perform;
 
	# Looking at the results...
	if ($retcode == 0) {
        	#print("Transfer went ok\n");
	        my $response_code = $curl->getinfo(CURLINFO_HTTP_CODE);
        	# judge result and next action based on $response_code
	        #print("Received response: $response_body\n");
                my $HA = decode_json($response_body);
                if ($HA->{"status"} eq "bad") {
                    if ($SYSLOG == 1) { syslog($facility, "Something bad. Agent: [".$host."] doesn't have interception: [".$uuid."]. Recreate!");}           
                    delete_interception($protocol, $host, $port, $path, $uuid);               
                    post_interception($protocol, $host, $port, $path, $uuid, $data);               
                }                	          	        	        
                else {
                    if ($SYSLOG == 1) { syslog($facility, "Interception [".$uuid."] exists on agent: [".$host."]");}
                    if($FORCE == 1) {
                        if ($SYSLOG == 1) { syslog($facility, "All good but agent: [".$host."] forced to recreate interception: [".$uuid."]");}
                        delete_interception($protocol, $host, $port, $path, $uuid);               
                        post_interception($protocol, $host, $port, $path, $uuid, $data);                               
                    }                    
                }
                
	} else {
                if ($SYSLOG == 1) { syslog($facility, "bad response during agent: [".$host."] check! interception: [".$uuid."]. Code:[".$retcode."]");}
	}
}


#check_interception($protocol,$host, $port, $path, $key, $IRECORDS{$key});
sub delete_interception {

	my $protocol = shift;
	my $host = shift;
	my $port = shift;
	my $path = shift;
	my $uuid = shift;

	my $curl = WWW::Curl::Easy->new;
 
	$curl->setopt(CURLOPT_HEADER,0);
	my $url = $protocol."://".$host.":".$port."".$path."/".$uuid;
	$curl->setopt(CURLOPT_URL, $url);
	$curl->setopt(CURLOPT_CUSTOMREQUEST, 'DELETE');
 
	# A filehandle, reference to a scalar or reference to a typeglob can be used here.
	my $response_body;
	open(my $fileb, ">", \$response_body);	
	$curl->setopt(CURLOPT_WRITEDATA,$fileb);
 
	# Starts the actual request
	my $retcode = $curl->perform;
 
	# Looking at the results...
	if ($retcode == 0) {
        	#print("Transfer went ok\n");
	        my $response_code = $curl->getinfo(CURLINFO_HTTP_CODE);
        	# judge result and next action based on $response_code
	        #print("Received response: $response_body\n");
                my $HA = decode_json($response_body);
                if ($HA->{"status"} eq "bad") {
                    if ($SYSLOG == 1) { syslog($facility, "couldn't delete interception for agent: [".$host."]. Interception: [".$uuid."]");}
                }                	          	        	        
                else {
                    if ($SYSLOG == 1) { syslog($facility, "Deleted interception for agent: [".$host."]. Interception: [".$uuid."]");}
                }
                
	} else {
	        if ($SYSLOG == 1) { syslog($facility, "bad response during agent: [".$host."] DELETE! interception: [".$uuid."]. Code:[".$retcode."]");}
	}
}

#check_interception($protocol,$host, $port, $path, $key, $IRECORDS{$key});
sub post_interception {

	my $protocol = shift;
	my $host = shift;
	my $port = shift;
	my $path = shift;
	my $uuid = shift;
	my $data = shift;

	my $curl = WWW::Curl::Easy->new;
 
	$curl->setopt(CURLOPT_HEADER,0);
	my $url = $protocol."://".$host.":".$port."/".$path;
	$curl->setopt(CURLOPT_URL, $url);
        $curl->setopt(CURLOPT_CUSTOMREQUEST, 'POST');
        $curl->setopt(CURLOPT_POSTFIELDS, $data);                                                                  
        my @headers  = ('Content-Type: application/json','Content-Length: ' . length($data));                                                                       
        $curl->setopt(CURLOPT_HTTPHEADER,\@headers);                    	         
	# A filehandle, reference to a scalar or reference to a typeglob can be used here.
	my $response_body;
	open(my $fileb, ">", \$response_body);	
	$curl->setopt(CURLOPT_WRITEDATA,$fileb);
 
	# Starts the actual request
	my $retcode = $curl->perform;
 
	# Looking at the results...
	if ($retcode == 0) {
        	#print("Transfer went ok\n");
	        my $response_code = $curl->getinfo(CURLINFO_HTTP_CODE);
        	# judge result and next action based on $response_code
	        #print("Received response: $response_body\n");
                my $HA = decode_json($response_body);
                if ($HA->{"status"} eq "bad") {
                    if ($SYSLOG == 1) { syslog($facility, "couldn't create interception for agent: [".$host."]. Interception: [".$uuid."]");}
                }                	          	        	        
                else {
                    if ($SYSLOG == 1) { syslog($facility, "Created interception for agent: [".$host."]. Interception: [".$uuid."]");}
                }
                
	} else {
	        if ($SYSLOG == 1) { syslog($facility, "bad response during  agent: [".$host."] POST! interception: [".$uuid."]. Code:[".$retcode."]");}
	}
}
 
 
1;
